@@ -1,5 +1,6 @@
 """Darker - apply black reformatting to only areas edited since the last commit"""
 
+import concurrent.futures
 import logging
 import sys
 import warnings
@@ -17,8 +18,9 @@ from darker.black_diff import (
 )
 from darker.chooser import choose_lines
 from darker.command_line import parse_command_line
-from darker.config import OutputMode, dump_config
-from darker.diff import diff_and_get_opcodes, opcodes_to_chunks
+from darker.concurrency import get_executor
+from darker.config import Exclusions, OutputMode, dump_config
+from darker.diff import diff_chunks
 from darker.exceptions import DependencyError, MissingPackageError
 from darker.git import (
     PRE_COMMIT_FROM_TO_REFS,
@@ -26,31 +28,39 @@ from darker.git import (
     EditedLinenumsDiffer,
     RevisionRange,
     get_missing_at_revision,
-    get_rev1_path,
+    get_path_in_repo,
     git_get_content_at_revision,
     git_get_modified_python_files,
     git_is_repository,
 )
 from darker.help import ISORT_INSTRUCTION
-from darker.highlighting import colorize
+from darker.highlighting import colorize, should_use_color
 from darker.import_sorting import apply_isort, isort
 from darker.linting import run_linters
-from darker.utils import GIT_DATEFORMAT, TextDocument, debug_dump, get_common_root
+from darker.utils import (
+    GIT_DATEFORMAT,
+    TextDocument,
+    debug_dump,
+    get_common_root,
+    glob_any,
+)
 from darker.verification import ASTVerifier, BinarySearch, NotEquivalentError
 
 logger = logging.getLogger(__name__)
 
+ProcessedDocument = Tuple[Path, TextDocument, TextDocument]
+
 
 def format_edited_parts(
     root: Path,
     changed_files: Collection[Path],  # pylint: disable=unsubscriptable-object
-    black_exclude: Collection[Path],  # pylint: disable=unsubscriptable-object
+    exclude: Exclusions,
     revrange: RevisionRange,
-    enable_isort: bool,
     black_config: BlackConfig,
     report_unmodified: bool,
-) -> Generator[Tuple[Path, TextDocument, TextDocument], None, None]:
-    """Black (and optional isort) formatting for chunks with edits since the last commit
+    workers: int = 1,
+) -> Generator[ProcessedDocument, None, None]:
+    """Black (and optional isort) formatting modified chunks in a set of files
 
     Files inside given directories and excluded by Black's configuration are not
     reformatted using Black, but their imports are still sorted. Also, linters will be
@@ -61,86 +71,135 @@ def format_edited_parts(
     :param root: The common root of all files to reformat
     :param changed_files: Python files and explicitly requested files which have been
                           modified in the repository between the given Git revisions
-    :param black_exclude: Python files to not reformat using Black, according to Black
-                          configuration
+    :param exclude: Files to exclude when running Black, and when running ``isort``
     :param revrange: The Git revisions to compare
-    :param enable_isort: ``True`` to also run ``isort`` first on each changed file
     :param black_config: Configuration to use for running Black
     :param report_unmodified: ``True`` to yield also files which weren't modified
+    :param workers: number of cpu processes to use (0 - autodetect)
     :return: A generator which yields details about changes for each file which should
              be reformatted, and skips unchanged files.
 
     """
-    for path_in_repo in sorted(changed_files):
-        src = root / path_in_repo
-        rev2_content = git_get_content_at_revision(path_in_repo, revrange.rev2, root)
-
-        # 1. run isort
-        if enable_isort:
-            rev2_isorted = apply_isort(
-                rev2_content,
-                src,
-                black_config.get("config"),
-                black_config.get("line_length"),
-            )
-        else:
-            rev2_isorted = rev2_content
-        if path_in_repo not in black_exclude:
-            # 9. A re-formatted Python file which produces an identical AST was
-            #    created successfully - write an updated file or print the diff if
-            #    there were any changes to the original
-            content_after_reformatting = _reformat_single_file(
+    with get_executor(max_workers=workers) as executor:
+        # pylint: disable=unsubscriptable-object
+        futures: List[concurrent.futures.Future[ProcessedDocument]] = []
+        edited_linenums_differ = EditedLinenumsDiffer(root, revrange)
+        for relative_path_in_rev2 in sorted(changed_files):
+            future = executor.submit(
+                _isort_and_blacken_single_file,
                 root,
-                path_in_repo,
+                relative_path_in_rev2,
+                edited_linenums_differ,
+                exclude,
                 revrange,
-                rev2_content,
-                rev2_isorted,
-                enable_isort,
                 black_config,
             )
-        else:
-            # File was excluded by Black configuration, don't reformat
-            content_after_reformatting = rev2_isorted
-        if report_unmodified or content_after_reformatting != rev2_content:
-            yield (src, rev2_content, content_after_reformatting)
+            futures.append(future)
+
+        for future in concurrent.futures.as_completed(futures):
+            (
+                absolute_path_in_rev2,
+                rev2_content,
+                content_after_reformatting,
+            ) = future.result()
+            if report_unmodified or content_after_reformatting != rev2_content:
+                yield (absolute_path_in_rev2, rev2_content, content_after_reformatting)
 
 
-def _reformat_single_file(  # pylint: disable=too-many-arguments,too-many-locals
+def _isort_and_blacken_single_file(  # pylint: disable=too-many-arguments
     root: Path,
-    relative_path: Path,
+    relative_path_in_rev2: Path,
+    edited_linenums_differ: EditedLinenumsDiffer,
+    exclude: Exclusions,
     revrange: RevisionRange,
+    black_config: BlackConfig,
+) -> ProcessedDocument:
+    """Black and/or isort formatting for modified chunks in a single file
+
+    :param root: Root directory for the relative path
+    :param relative_path_in_rev2: Relative path to a Python source code file
+    :param exclude: Files to exclude when running Black, and when running ``isort``
+    :param revrange: The Git revisions to compare
+    :param black_config: Configuration to use for running Black
+    :return: Details about changes for the file
+
+    """
+    # With VSCode, `relative_path_in_rev2` may be a `.py.<HASH>.tmp` file in the
+    # working tree instead of a `.py` file.
+    absolute_path_in_rev2 = root / relative_path_in_rev2
+    rev2_content = git_get_content_at_revision(
+        relative_path_in_rev2, revrange.rev2, root
+    )
+    # 1. run isort
+    rev2_isorted = apply_isort(
+        rev2_content,
+        relative_path_in_rev2,
+        exclude.isort,
+        edited_linenums_differ,
+        black_config.get("config"),
+        black_config.get("line_length"),
+    )
+    has_isort_changes = rev2_isorted != rev2_content
+    if not glob_any(relative_path_in_rev2, exclude.black):
+        # 9. A re-formatted Python file which produces an identical AST was
+        #    created successfully - write an updated file or print the diff if
+        #    there were any changes to the original
+        content_after_reformatting = _blacken_single_file(
+            root,
+            relative_path_in_rev2,
+            get_path_in_repo(relative_path_in_rev2),
+            edited_linenums_differ,
+            rev2_content,
+            rev2_isorted,
+            has_isort_changes,
+            black_config,
+        )
+    else:
+        # File was excluded by Black configuration, don't reformat
+        content_after_reformatting = rev2_isorted
+    return absolute_path_in_rev2, rev2_content, content_after_reformatting
+
+
+def _blacken_single_file(  # pylint: disable=too-many-arguments,too-many-locals
+    root: Path,
+    relative_path_in_rev2: Path,
+    relative_path_in_repo: Path,
+    edited_linenums_differ: EditedLinenumsDiffer,
     rev2_content: TextDocument,
     rev2_isorted: TextDocument,
-    enable_isort: bool,
+    has_isort_changes: bool,
     black_config: BlackConfig,
 ) -> TextDocument:
     """In a Python file, reformat chunks with edits since the last commit using Black
 
     :param root: The common root of all files to reformat
-    :param relative_path: Relative path to a Python source code file
-    :param revrange: The Git revisions to compare
+    :param relative_path_in_rev2: Relative path to a Python source code file. Possibly a
+                                  VSCode ``.py.<HASH>.tmp`` file in the working tree.
+    :param relative_path_in_repo: Relative path to source in the Git repository. Same as
+                                  ``relative_path_in_rev2`` save for VSCode temp files.
+    :param edited_linenums_differ: Helper for finding out which lines were edited
     :param rev2_content: Contents of the file at ``revrange.rev2``
     :param rev2_isorted: Contents of the file after optional import sorting
-    :param enable_isort: ``True`` if ``isort`` was already run for the file
+    :param has_isort_changes: ``True`` if ``isort`` was run and modified the file
     :param black_config: Configuration to use for running Black
     :return: Contents of the file after reformatting
     :raise: NotEquivalentError
 
     """
-    src = root / relative_path
-    rev1_relative_path = get_rev1_path(relative_path)
-    edited_linenums_differ = EditedLinenumsDiffer(root, revrange)
+    absolute_path_in_rev2 = root / relative_path_in_rev2
 
     # 4. run black
     formatted = run_black(rev2_isorted, black_config)
-    logger.debug("Read %s lines from edited file %s", len(rev2_isorted.lines), src)
+    logger.debug(
+        "Read %s lines from edited file %s",
+        len(rev2_isorted.lines),
+        absolute_path_in_rev2,
+    )
     logger.debug("Black reformat resulted in %s lines", len(formatted.lines))
 
     # 5. get the diff between the edited and reformatted file
-    opcodes = diff_and_get_opcodes(rev2_isorted, formatted)
-
     # 6. convert the diff into chunks
-    black_chunks = list(opcodes_to_chunks(opcodes, rev2_isorted, formatted))
+    black_chunks = diff_chunks(rev2_isorted, formatted)
 
     # Exit early if nothing to do
     if not black_chunks:
@@ -158,15 +217,15 @@ def _reformat_single_file(  # pylint: disable=too-many-arguments,too-many-locals
             logger.debug(
                 "Trying with %s lines of context for `git diff -U %s`",
                 context_lines,
-                src,
+                absolute_path_in_rev2,
             )
         # 2. diff the given revision and worktree for the file
         # 3. extract line numbers in the edited to-file for changed lines
         edited_linenums = edited_linenums_differ.revision_vs_lines(
-            rev1_relative_path, rev2_isorted, context_lines
+            relative_path_in_repo, rev2_isorted, context_lines
         )
-        if enable_isort and not edited_linenums and rev2_isorted == rev2_content:
-            logger.debug("No changes in %s after isort", src)
+        if has_isort_changes and not edited_linenums:
+            logger.debug("No changes in %s after isort", absolute_path_in_rev2)
             last_successful_reformat = rev2_isorted
             break
 
@@ -189,7 +248,7 @@ def _reformat_single_file(  # pylint: disable=too-many-arguments,too-many-locals
             debug_dump(black_chunks, edited_linenums)
             logger.debug(
                 "AST verification of %s with %s lines of context failed",
-                src,
+                absolute_path_in_rev2,
                 context_lines,
             )
             minimum_context_lines.respond(False)
@@ -197,7 +256,7 @@ def _reformat_single_file(  # pylint: disable=too-many-arguments,too-many-locals
             minimum_context_lines.respond(True)
             last_successful_reformat = chosen
     if not last_successful_reformat:
-        raise NotEquivalentError(relative_path)
+        raise NotEquivalentError(relative_path_in_rev2)
     return last_successful_reformat
 
 
@@ -208,7 +267,11 @@ def modify_file(path: Path, new_content: TextDocument) -> None:
 
 
 def print_diff(
-    path: Path, old: TextDocument, new: TextDocument, root: Path = None
+    path: Path,
+    old: TextDocument,
+    new: TextDocument,
+    root: Path,
+    use_color: bool,
 ) -> None:
     """Print ``black --diff`` style output for the changes
 
@@ -220,8 +283,6 @@ def print_diff(
     Modification times should be in the format "YYYY-MM-DD HH:MM:SS:mmmmmm +0000"
 
     """
-    if root is None:
-        root = Path.cwd()
     relative_path = path.resolve().relative_to(root).as_posix()
     diff = "\n".join(
         line.rstrip("\n")
@@ -235,12 +296,12 @@ def print_diff(
             n=5,  # Black shows 5 lines of context, do the same
         )
     )
-    print(colorize(diff, "diff"))
+    print(colorize(diff, "diff", use_color))
 
 
-def print_source(new: TextDocument) -> None:
+def print_source(new: TextDocument, use_color: bool) -> None:
     """Print the reformatted Python source code"""
-    if sys.stdout.isatty():
+    if use_color:
         try:
             (
                 highlight,
@@ -379,28 +440,37 @@ def main(argv: List[str] = None) -> int:
                 path.relative_to(root) for path in files_to_process
             }
         black_exclude = {
-            f for f in changed_files_to_process if root / f not in files_to_blacken
+            str(path)
+            for path in changed_files_to_process
+            if root / path not in files_to_blacken
         }
 
+    use_color = should_use_color(config["color"])
     formatting_failures_on_modified_lines = False
-    for path, old, new in format_edited_parts(
-        root,
-        changed_files_to_process,
-        black_exclude,
-        revrange,
-        args.isort,
-        black_config,
-        report_unmodified=output_mode == OutputMode.CONTENT,
+    for path, old, new in sorted(
+        format_edited_parts(
+            root,
+            changed_files_to_process,
+            Exclusions(black=black_exclude, isort=set() if args.isort else {"**/*"}),
+            revrange,
+            black_config,
+            report_unmodified=output_mode == OutputMode.CONTENT,
+            workers=config["workers"],
+        ),
     ):
         formatting_failures_on_modified_lines = True
         if output_mode == OutputMode.DIFF:
-            print_diff(path, old, new, root)
+            print_diff(path, old, new, root, use_color)
         elif output_mode == OutputMode.CONTENT:
-            print_source(new)
+            print_source(new, use_color)
         if write_modified_files:
             modify_file(path, new)
     linter_failures_on_modified_lines = run_linters(
-        args.lint, root, changed_files_to_process, revrange
+        args.lint,
+        root,
+        changed_files_to_process,
+        revrange,
+        use_color,
     )
     return (
         1
