@@ -55,7 +55,8 @@ class OptionsForReadmeAction(Action):
         optional_arguments_group = next(
             group
             for group in parser._action_groups
-            if group.title == "optional arguments"
+            # The group title for options differs between Python versions
+            if group.title in {"optional arguments", "options"}
         )
         actions = []
         for action in optional_arguments_group._group_actions:
@@ -101,8 +102,6 @@ class LogLevelAction(Action):  # pylint: disable=too-few-public-methods
         values: Union[str, Sequence[Any], None],
         option_string: str = None,
     ) -> None:
-        assert isinstance(values, list)
-        assert all(isinstance(v, str) for v in values)
         current_level = getattr(namespace, self.dest, self.default)
         new_level = current_level + self.const
         new_level = max(new_level, logging.DEBUG)
