@@ -1,7 +1,7 @@
 """Command line parsing for the ``darker`` binary"""
 
 from argparse import SUPPRESS, ArgumentParser, Namespace
-from typing import Any, List, Optional, Text, Tuple
+from typing import Any, List, Optional, Tuple
 
 from darker import help as hlp
 from darker.argparse_helpers import (
@@ -15,6 +15,7 @@ from darker.config import (
     get_effective_config,
     get_modified_config,
     load_config,
+    override_color_with_environment,
 )
 from darker.version import __version__
 
@@ -31,7 +32,7 @@ def make_argument_parser(require_src: bool) -> ArgumentParser:
     )
     parser.register("action", "log_level", LogLevelAction)
 
-    def add_arg(help_text: Optional[Text], *name_or_flags: Text, **kwargs: Any) -> None:
+    def add_arg(help_text: Optional[str], *name_or_flags: str, **kwargs: Any) -> None:
         kwargs["help"] = help_text
         parser.add_argument(*name_or_flags, **kwargs)
 
@@ -52,6 +53,8 @@ def make_argument_parser(require_src: bool) -> ArgumentParser:
         const=-10,
     )
     add_arg(hlp.QUIET, "-q", "--quiet", action="log_level", dest="log_level", const=10)
+    add_arg(hlp.COLOR, "--color", action="store_const", dest="color", const=True)
+    add_arg(hlp.NO_COLOR, "--no-color", action="store_const", dest="color", const=False)
     add_arg(hlp.VERSION, "--version", action="version", version=__version__)
     add_arg(
         hlp.SKIP_STRING_NORMALIZATION,
@@ -82,6 +85,7 @@ def make_argument_parser(require_src: bool) -> ArgumentParser:
         dest="line_length",
         metavar="LENGTH",
     )
+    add_arg(hlp.WORKERS, "-W", "--workers", type=int, dest="workers", default=1)
     # A hidden option for printing command lines option in a format suitable for
     # `README.rst`:
     add_arg(SUPPRESS, "--options-for-readme", action=OptionsForReadmeAction)
@@ -92,8 +96,8 @@ def parse_command_line(argv: List[str]) -> Tuple[Namespace, DarkerConfig, Darker
     """Return the parsed command line, using defaults from a configuration file
 
     Also return the effective configuration which combines defaults, the configuration
-    read from ``pyproject.toml`` (or the path given in ``--config``), and command line
-    arguments.
+    read from ``pyproject.toml`` (or the path given in ``--config``), environment
+    variables, and command line arguments.
 
     Finally, also return the set of configuration options which differ from defaults.
 
@@ -102,9 +106,11 @@ def parse_command_line(argv: List[str]) -> Tuple[Namespace, DarkerConfig, Darker
     parser_for_srcs = make_argument_parser(require_src=False)
     args = parser_for_srcs.parse_args(argv)
 
-    # 2. Locate `pyproject.toml` based on those paths, or in the current directory if no
-    #    paths were given. Load Darker configuration from it.
-    config = load_config(args.src)
+    # 2. Locate `pyproject.toml` based on the `-c`/`--config` command line option, or
+    #    if it's not provided, based on the paths to process, or in the current
+    #    directory if no paths were given. Load Darker configuration from it.
+    pyproject_config = load_config(args.config, args.src)
+    config = override_color_with_environment(pyproject_config)
 
     # 3. Use configuration as defaults for re-parsing command line arguments, and don't
     #    require file/directory paths if they are specified in configuration.
