@@ -1,10 +1,12 @@
 """Load and save configuration in TOML format"""
 
 import logging
+import os
 import sys
 from argparse import ArgumentParser, Namespace
+from dataclasses import dataclass
 from pathlib import Path
-from typing import Iterable, List, cast
+from typing import Collection, Iterable, List, Optional, cast
 
 import toml
 
@@ -36,9 +38,11 @@ class DarkerConfig(TypedDict, total=False):
     lint: List[str]
     config: str
     log_level: int
+    color: bool
     skip_string_normalization: bool
     skip_magic_trailing_comma: bool
     line_length: int
+    workers: int
 
 
 class OutputMode:
@@ -96,23 +100,59 @@ def validate_config_output_mode(config: DarkerConfig) -> None:
     )
 
 
-def load_config(srcs: Iterable[str]) -> DarkerConfig:
-    """Find and load Darker configuration from given path or pyproject.toml
+def override_color_with_environment(pyproject_config: DarkerConfig) -> DarkerConfig:
+    """Override ``color`` if the ``PY_COLORS`` environment variable is '0' or '1'
 
-    :param srcs: File(s) and directory/directories which will be processed. Their paths
-                 are used to look for the ``pyproject.toml`` configuration file.
+    :param config: The configuration read from ``pyproject.toml``
+    :return: The modified configuration
 
     """
-    path = find_project_root(tuple(srcs or ["."])) / "pyproject.toml"
-    if path.is_file():
-        pyproject_toml = toml.load(path)
-        config = cast(
-            DarkerConfig, pyproject_toml.get("tool", {}).get("darker", {}) or {}
-        )
-        replace_log_level_name(config)
-        validate_config_output_mode(config)
-        return config
-    return {}
+    config = pyproject_config.copy()
+    py_colors = os.getenv("PY_COLORS")
+    if py_colors in {"0", "1"}:
+        config["color"] = py_colors == "1"
+    elif os.getenv("NO_COLOR") is not None:
+        config["color"] = False
+    elif os.getenv("FORCE_COLOR") is not None:
+        config["color"] = True
+    return config
+
+
+def load_config(path: Optional[str], srcs: Iterable[str]) -> DarkerConfig:
+    """Find and load Darker configuration from a TOML configuration file
+
+    Darker determines the location for the configuration file by trying the following:
+    - the file path in the `path` argument, given using the ``-c``/``--config`` command
+      line option
+    - ``pyproject.toml`` inside the directory specified by the `path` argument
+    - ``pyproject.toml`` from a common parent directory to all items in `srcs`
+    - ``pyproject.toml`` in the current working directory if `srcs` is empty
+
+    :param path: The file or directory specified using the ``-c``/``--config`` command
+                 line option, or `None` if the option was omitted.
+    :param srcs: File(s) and directory/directories to be processed by Darker.
+
+    """
+    if path:
+        for candidate_path in [Path(path), Path(path, "pyproject.toml")]:
+            if candidate_path.is_file():
+                config_path = candidate_path
+                break
+        else:
+            if Path(path).is_dir() or path.endswith(os.sep):
+                raise ConfigurationError(
+                    f"Configuration file {Path(path, 'pyproject.toml')} not found"
+                )
+            raise ConfigurationError(f"Configuration file {path} not found")
+    else:
+        config_path = find_project_root(tuple(srcs or ["."])) / "pyproject.toml"
+        if not config_path.is_file():
+            return {}
+    pyproject_toml = toml.load(config_path)
+    config = cast(DarkerConfig, pyproject_toml.get("tool", {}).get("darker", {}) or {})
+    replace_log_level_name(config)
+    validate_config_output_mode(config)
+    return config
 
 
 def get_effective_config(args: Namespace) -> DarkerConfig:
@@ -141,3 +181,23 @@ def dump_config(config: DarkerConfig) -> str:
     """Return the configuration in TOML format"""
     dump = toml.dumps(config, encoder=TomlArrayLinesEncoder())
     return f"[tool.darker]\n{dump}"
+
+
+@dataclass
+class Exclusions:
+    """File exclusions patterns for pre-processing steps
+
+    For each pre-processor, there is a collection of glob patterns. When running Darker,
+    any files matching at least one of the glob patterns is supposed to be skipped when
+    running the corresponding pre-processor. If the collection of glob patterns is
+    empty, the pre-processor is run for all files.
+
+    The pre-processors whose exclusion lists are currently stored in this data
+    structure are
+    - Black
+    - ``isort``
+
+    """
+
+    black: Collection[str]
+    isort: Collection[str]
