@@ -66,8 +66,9 @@ a mixed result with only selected regions reformatted can be reconstructed.
 
 import logging
 from difflib import SequenceMatcher
-from typing import Generator, List, Tuple
+from typing import Generator, List, Sequence, Tuple
 
+from darker.multiline_strings import find_overlap
 from darker.utils import DiffChunk, TextDocument
 
 logger = logging.getLogger(__name__)
@@ -101,32 +102,59 @@ def diff_and_get_opcodes(
 
 def _validate_opcodes(opcodes: List[Tuple[str, int, int, int, int]]) -> None:
     """Make sure every other opcode is an 'equal' tag"""
-    assert all(
+    if not all(
         (tag1 == "equal") != (tag2 == "equal")
         for (tag1, _, _, _, _), (tag2, _, _, _, _) in zip(opcodes[:-1], opcodes[1:])
-    ), opcodes
+    ):
+        raise ValueError(f"Unexpected opcodes in {opcodes!r}")
 
 
 def opcodes_to_edit_linenums(
-    opcodes: List[Tuple[str, int, int, int, int]], context_lines: int
+    opcodes: List[Tuple[str, int, int, int, int]],
+    context_lines: int,
+    multiline_string_ranges: Sequence[Tuple[int, int]],
 ) -> Generator[int, None, None]:
     """Convert diff opcodes to line numbers of edited lines in the destination file
 
-    :param opcodes: The diff opcodes to convert
+    On top of a straight mapping from line ranges to individual line numbers, this
+    function extends each diff opcode line range
+    - upwards and downwards for as many lines as determined by ``context_lines``
+    - to make sure the range covers any multiline strings completely
+
+    :param opcodes: The diff opcodes to convert. 0-based, end-exclusive.
     :param context_lines: The number of lines before and after an edited line to mark
                           edited as well
+    :param multiline_string_ranges: Line ranges of multi-line strings. 1-based,
+                                    end-exclusive.
+    :return: Generates a list of integer 1-based line numbers
 
     """
     if not opcodes:
         return
     _validate_opcodes(opcodes)
+
+    # Calculate the last line number beyond which we won't extend with extra context
+    # lines
+    _tag, _i1, _i2, _j1, last_opcode_end = opcodes[-1]
+    _, last_multiline_string_end = (
+        multiline_string_ranges[-1] if multiline_string_ranges else (None, 0)
+    )
+    lastline = max(last_opcode_end + 1, last_multiline_string_end)
+
     prev_chunk_end = 1
-    _tag, _i1, _i2, _j1, end = opcodes[-1]
     for tag, _i1, _i2, j1, j2 in opcodes:
-        if tag != "equal":
-            chunk_end = min(j2 + 1 + context_lines, end + 1)
-            yield from range(max(j1 + 1 - context_lines, prev_chunk_end), chunk_end)
-            prev_chunk_end = chunk_end
+        if tag == "equal":
+            continue
+        chunk_start = j1 + 1 - context_lines
+        chunk_end = j2 + 1 + context_lines
+        multiline_string_range = find_overlap(
+            chunk_start, chunk_end, multiline_string_ranges
+        )
+        if multiline_string_range:
+            chunk_start = min(chunk_start, multiline_string_range[0])
+            chunk_end = max(chunk_end, multiline_string_range[1])
+        yield from range(max(chunk_start, prev_chunk_end), min(chunk_end, lastline))
+        prev_chunk_end = chunk_end
 
 
 def opcodes_to_chunks(
@@ -147,5 +175,31 @@ def opcodes_to_chunks(
 
     """
     _validate_opcodes(opcodes)
-    for tag, i1, i2, j1, j2 in opcodes:
-        yield i1 + 1, src.lines[i1:i2], dst.lines[j1:j2]
+    for _tag, src_start, src_end, dst_start, dst_end in opcodes:
+        yield src_start + 1, src.lines[src_start:src_end], dst.lines[dst_start:dst_end]
+
+
+def diff_chunks(src: TextDocument, dst: TextDocument) -> List[DiffChunk]:
+    """Diff two documents and return the list of chunks in the diff
+
+    Each chunk is a 3-tuple::
+
+        (
+            linenum: int,
+            old_lines: List[str],
+            new_lines: List[str],
+        )
+
+    ``old_lines`` and ``new_lines`` may be
+
+    - identical to indicate a chunk with no changes,
+    - of the same length but different items to indicate some modified lines, or
+    - of different lengths to indicate removed or inserted lines.
+
+    For the return value ``retval``, the following always holds::
+
+        retval[n + 1][0] == retval[n][0] + len(retval[n][old_lines])
+
+    """
+    opcodes = diff_and_get_opcodes(src, dst)
+    return list(opcodes_to_chunks(opcodes, src, dst))
