@@ -8,10 +8,11 @@ from dataclasses import dataclass
 from datetime import datetime
 from functools import lru_cache
 from pathlib import Path
-from subprocess import DEVNULL, PIPE, CalledProcessError, check_output, run
-from typing import Dict, Iterable, List, Set, Tuple
+from subprocess import DEVNULL, PIPE, CalledProcessError, check_output, run  # nosec
+from typing import Dict, Iterable, List, Optional, Set, Tuple, Union, overload
 
 from darker.diff import diff_and_get_opcodes, opcodes_to_edit_linenums
+from darker.multiline_strings import get_multiline_string_ranges
 from darker.utils import GIT_DATEFORMAT, TextDocument
 
 logger = logging.getLogger(__name__)
@@ -69,17 +70,18 @@ def git_get_content_at_revision(path: Path, revision: str, cwd: Path) -> TextDoc
     :param cwd: The root of the Git repository
 
     """
-    assert (
-        not path.is_absolute()
-    ), f"the 'path' parameter must receive a relative path, got {path!r} instead"
+    if path.is_absolute():
+        raise ValueError(
+            f"the 'path' parameter must receive a relative path, got {path!r} instead"
+        )
 
     if revision == WORKTREE:
         abspath = cwd / path
         return TextDocument.from_file(abspath)
     cmd = ["show", f"{revision}:./{path.as_posix()}"]
     try:
-        return TextDocument.from_lines(
-            _git_check_output_lines(cmd, cwd, exit_on_error=False),
+        return TextDocument.from_bytes(
+            _git_check_output(cmd, cwd, exit_on_error=False),
             mtime=git_get_mtime_at_commit(path, revision, cwd),
         )
     except CalledProcessError as exc_info:
@@ -168,7 +170,7 @@ class RevisionRange:
         return cls(rev1 if common_ancestor == rev1_hash else common_ancestor, rev2)
 
 
-def get_rev1_path(path: Path) -> Path:
+def get_path_in_repo(path: Path) -> Path:
     """Return the relative path to the file in the old revision
 
     This is usually the same as the relative path on the command line. But in the
@@ -186,7 +188,7 @@ def get_rev1_path(path: Path) -> Path:
 
 
 def should_reformat_file(path: Path) -> bool:
-    return path.exists() and get_rev1_path(path).suffix == ".py"
+    return path.exists() and get_path_in_repo(path).suffix == ".py"
 
 
 @lru_cache(maxsize=1)
@@ -206,15 +208,45 @@ def _git_check_output_lines(
     cmd: List[str], cwd: Path, exit_on_error: bool = True
 ) -> List[str]:
     """Log command line, run Git, split stdout to lines, exit with 123 on error"""
+    return _git_check_output(
+        cmd,
+        cwd,
+        exit_on_error=exit_on_error,
+        encoding="utf-8",
+    ).splitlines()
+
+
+@overload
+def _git_check_output(
+    cmd: List[str], cwd: Path, *, exit_on_error: bool = ..., encoding: None = ...
+) -> bytes:
+    ...
+
+
+@overload
+def _git_check_output(
+    cmd: List[str], cwd: Path, *, exit_on_error: bool = ..., encoding: str
+) -> str:
+    ...
+
+
+def _git_check_output(
+    cmd: List[str],
+    cwd: Path,
+    *,
+    exit_on_error: bool = True,
+    encoding: Optional[str] = None,
+) -> Union[str, bytes]:
+    """Log command line, run Git, return stdout, exit with 123 on error"""
     logger.debug("[%s]$ git %s", cwd, " ".join(cmd))
     try:
-        return check_output(
+        return check_output(  # nosec
             ["git"] + cmd,
             cwd=str(cwd),
-            encoding="utf-8",
+            encoding=encoding,
             stderr=PIPE,
             env=_make_git_env(),
-        ).splitlines()
+        )
     except CalledProcessError as exc_info:
         if not exit_on_error:
             raise
@@ -239,11 +271,13 @@ def _git_exists_in_revision(path: Path, rev2: str, cwd: Path) -> bool:
              it doesn't.
 
     """
+    if (cwd / path).resolve() == cwd.resolve():
+        return True
     # Surprise: On Windows, `git cat-file` doesn't work with backslash directory
     # separators in paths. We need to use Posix paths and forward slashes instead.
     cmd = ["git", "cat-file", "-e", f"{rev2}:{path.as_posix()}"]
     logger.debug("[%s]$ %s", cwd, " ".join(cmd))
-    result = run(
+    result = run(  # nosec
         cmd,
         cwd=str(cwd),
         check=False,
@@ -343,6 +377,66 @@ def git_get_modified_python_files(
     return {path for path in changed_paths if should_reformat_file(cwd / path)}
 
 
+def _revision_vs_lines(
+    root: Path, path_in_repo: Path, rev1: str, content: TextDocument, context_lines: int
+) -> List[int]:
+    """Return changed line numbers between the given revision and given text content
+
+    The revision range was provided when this `EditedLinenumsDiffer` object was
+    created. Content for the given `path_in_repo` at `rev1` of that revision is
+    taken from the Git repository, and the provided text content is compared against
+    that historical version.
+
+    The actual implementation is here instead of in
+    `EditedLinenumsDiffer._revision_vs_lines` so it is accessibe both as a method and as
+    a module global for use in the `_compare_revisions` function.
+
+    :param root: Root directory for the relative path `path_in_repo`
+    :param path_in_repo: Path of the file to compare, relative to repository root
+    :param rev1: The Git revision to compare the on-disk worktree version to
+    :param content: The contents to compare to, e.g. from current working tree
+    :param context_lines: The number of lines to include before and after a change
+    :return: Line numbers of lines changed between the revision and given content
+
+    """
+    old = git_get_content_at_revision(path_in_repo, rev1, root)
+    edited_opcodes = diff_and_get_opcodes(old, content)
+    multiline_string_ranges = get_multiline_string_ranges(content)
+    return list(
+        opcodes_to_edit_linenums(edited_opcodes, context_lines, multiline_string_ranges)
+    )
+
+
+@lru_cache(maxsize=1)
+def _compare_revisions(
+    root: Path, path_in_repo: Path, rev1: str, context_lines: int
+) -> List[int]:
+    """Get line numbers of lines changed between a given revision and the worktree
+
+    Also includes `context_lines` number of extra lines before and after each
+    modified line.
+
+    The actual implementation is here instead of in
+    `EditedLinenumsDiffer.compare_revisions` because `lru_cache` leaks memory when used
+    on methods. See https://stackoverflow.com/q/33672412/15770
+
+    :param root: Root directory for the relative path `path_in_repo`
+    :param path_in_repo: Path of the file to compare, relative to repository root
+    :param rev1: The Git revision to compare the on-disk worktree version to
+    :param context_lines: The number of lines to include before and after a change
+    :return: Line numbers of lines changed between the revision and given content
+
+    """
+    content = TextDocument.from_file(root / path_in_repo)
+    linenums = _revision_vs_lines(root, path_in_repo, rev1, content, context_lines)
+    logger.debug(
+        "Edited line numbers in %s: %s",
+        path_in_repo,
+        " ".join(str(n) for n in linenums),
+    )
+    return linenums
+
+
 @dataclass(frozen=True)
 class EditedLinenumsDiffer:
     """Find out changed lines for a file between given Git revisions"""
@@ -350,22 +444,38 @@ class EditedLinenumsDiffer:
     root: Path
     revrange: RevisionRange
 
-    @lru_cache(maxsize=1)
     def compare_revisions(self, path_in_repo: Path, context_lines: int) -> List[int]:
-        """Return numbers of lines changed between a given revision and the worktree"""
-        content = TextDocument.from_file(self.root / path_in_repo)
-        linenums = self.revision_vs_lines(path_in_repo, content, context_lines)
-        logger.debug(
-            "Edited line numbers in %s: %s",
-            path_in_repo,
-            " ".join(str(n) for n in linenums),
+        """Get line numbers of lines changed between a given revision and the worktree
+
+        Also includes `context_lines` number of extra lines before and after each
+        modified line.
+
+        The actual implementation is in the module global `_compare_revisions` function
+        because `lru_cache` leaks memory when used on methods.
+        See https://stackoverflow.com/q/33672412/15770
+
+        :param path_in_repo: Path of the file to compare, relative to repository root
+        :param context_lines: The number of lines to include before and after a change
+        :return: Line numbers of lines changed between the revision and given content
+
+        """
+        return _compare_revisions(
+            self.root, path_in_repo, self.revrange.rev1, context_lines
         )
-        return linenums
 
     def revision_vs_lines(
         self, path_in_repo: Path, content: TextDocument, context_lines: int
     ) -> List[int]:
-        """For file `path_in_repo`, return changed line numbers from given revision
+        """Return changed line numbers between the given revision and given text content
+
+        The revision range was provided when this `EditedLinenumsDiffer` object was
+        created. Content for the given `path_in_repo` at `rev1` of that revision is
+        taken from the Git repository, and the provided text content is compared against
+        that historical version.
+
+        The actual implementation is in the module global `_revision_vs_lines` function
+        so this can be called from both as a method and the module global
+        `_compare_revisions` function.
 
         :param path_in_repo: Path of the file to compare, relative to repository root
         :param content: The contents to compare to, e.g. from current working tree
@@ -373,6 +483,6 @@ class EditedLinenumsDiffer:
         :return: Line numbers of lines changed between the revision and given content
 
         """
-        old = git_get_content_at_revision(path_in_repo, self.revrange.rev1, self.root)
-        edited_opcodes = diff_and_get_opcodes(old, content)
-        return list(opcodes_to_edit_linenums(edited_opcodes, context_lines))
+        return _revision_vs_lines(
+            self.root, path_in_repo, self.revrange.rev1, content, context_lines
+        )
