@@ -2,6 +2,7 @@
 
 """Unit tests for :mod:`darker.command_line` and :mod:`darker.__main__`"""
 
+import os
 import re
 from argparse import ArgumentError
 from importlib import reload
@@ -16,7 +17,7 @@ import darker.help
 from darker import black_diff
 from darker.__main__ import main
 from darker.command_line import make_argument_parser, parse_command_line
-from darker.config import ConfigurationError
+from darker.config import ConfigurationError, Exclusions
 from darker.git import RevisionRange
 from darker.tests.helpers import filter_dict, isort_present, raises_if_exception
 from darker.utils import TextDocument, joinlines
@@ -90,6 +91,36 @@ def test_parse_command_line_config_src(
         assert filter_dict(dict(modified_cfg), "src") == expect
 
 
+@pytest.mark.kwparametrize(
+    dict(argv=["."], expect="pylint"),
+    dict(argv=["./subdir/"], expect="flake8"),
+    dict(argv=["--config", "./pyproject.toml", "."], expect="pylint"),
+    dict(argv=["--config", "./subdir/pyproject.toml", "."], expect="flake8"),
+    dict(argv=["--config", "./pyproject.toml", "subdir/"], expect="pylint"),
+    dict(argv=["--config", "./subdir/pyproject.toml", "subdir/"], expect="flake8"),
+)
+def test_parse_command_line_config_location_specified(
+    tmp_path,
+    monkeypatch,
+    argv,
+    expect,
+):
+    """Darker configuration is read from file pointed to using ``-c``/``--config``"""
+    monkeypatch.chdir(tmp_path)
+    subdir = tmp_path / "subdir"
+    subdir.mkdir()
+    root_config = tmp_path / "pyproject.toml"
+    subdir_config = subdir / "pyproject.toml"
+    root_config.write_text(toml.dumps({"tool": {"darker": {"lint": "pylint"}}}))
+    subdir_config.write_text(toml.dumps({"tool": {"darker": {"lint": "flake8"}}}))
+
+    args, effective_cfg, modified_cfg = parse_command_line(argv)
+
+    assert args.lint == expect
+    assert effective_cfg["lint"] == expect
+    assert modified_cfg["lint"] == expect
+
+
 @pytest.mark.kwparametrize(
     dict(
         argv=["."],
@@ -211,6 +242,18 @@ def test_parse_command_line_config_src(
         expect_config=("config", "my.cfg"),
         expect_modified=("config", "my.cfg"),
     ),
+    dict(
+        argv=["-c", "subdir_with_config", "."],
+        expect_value=("config", "subdir_with_config"),
+        expect_config=("config", "subdir_with_config"),
+        expect_modified=("config", "subdir_with_config"),
+    ),
+    dict(
+        argv=["--config=subdir_with_config", "."],
+        expect_value=("config", "subdir_with_config"),
+        expect_config=("config", "subdir_with_config"),
+        expect_modified=("config", "subdir_with_config"),
+    ),
     dict(
         argv=["."],
         expect_value=("log_level", 30),
@@ -241,6 +284,69 @@ def test_parse_command_line_config_src(
         expect_config=("log_level", "CRITICAL"),
         expect_modified=("log_level", "CRITICAL"),
     ),
+    dict(
+        argv=["."],
+        environ={},
+        expect_value=("color", None),
+        expect_config=("color", None),
+        expect_modified=("color", ...),
+    ),
+    dict(
+        argv=["."],
+        environ={"PY_COLORS": "0"},
+        expect_value=("color", False),
+        expect_config=("color", False),
+        expect_modified=("color", False),
+    ),
+    dict(
+        argv=["."],
+        environ={"PY_COLORS": "1"},
+        expect_value=("color", True),
+        expect_config=("color", True),
+        expect_modified=("color", True),
+    ),
+    dict(
+        argv=["--color", "."],
+        environ={},
+        expect_value=("color", True),
+        expect_config=("color", True),
+        expect_modified=("color", True),
+    ),
+    dict(
+        argv=["--color", "."],
+        environ={"PY_COLORS": "0"},
+        expect_value=("color", True),
+        expect_config=("color", True),
+        expect_modified=("color", True),
+    ),
+    dict(
+        argv=["--color", "."],
+        environ={"PY_COLORS": "1"},
+        expect_value=("color", True),
+        expect_config=("color", True),
+        expect_modified=("color", True),
+    ),
+    dict(
+        argv=["--no-color", "."],
+        environ={},
+        expect_value=("color", False),
+        expect_config=("color", False),
+        expect_modified=("color", False),
+    ),
+    dict(
+        argv=["--no-color", "."],
+        environ={"PY_COLORS": "0"},
+        expect_value=("color", False),
+        expect_config=("color", False),
+        expect_modified=("color", False),
+    ),
+    dict(
+        argv=["--no-color", "."],
+        environ={"PY_COLORS": "1"},
+        expect_value=("color", False),
+        expect_config=("color", False),
+        expect_modified=("color", False),
+    ),
     dict(
         argv=["."],
         expect_value=("skip_string_normalization", None),
@@ -303,14 +409,20 @@ def test_parse_command_line_config_src(
         expect_config=("src", ["valid/path", "another/valid/path"]),
         expect_modified=("src", ["valid/path", "another/valid/path"]),
     ),
+    environ={},
 )
 def test_parse_command_line(
-    tmp_path, monkeypatch, argv, expect_value, expect_config, expect_modified
+    tmp_path, monkeypatch, argv, environ, expect_value, expect_config, expect_modified
 ):
     """``parse_command_line()`` parses options correctly"""
     monkeypatch.chdir(tmp_path)
     (tmp_path / "dummy.py").touch()
-    with raises_if_exception(expect_value) as expect_exception:
+    (tmp_path / "my.cfg").touch()
+    (tmp_path / "subdir_with_config").mkdir()
+    (tmp_path / "subdir_with_config" / "pyproject.toml").touch()
+    with patch.dict(os.environ, environ, clear=True), raises_if_exception(
+        expect_value
+    ) as expect_exception:
 
         args, effective_cfg, modified_cfg = parse_command_line(argv)
 
@@ -512,9 +624,8 @@ def test_black_options_skip_magic_trailing_comma(git_repo, config, options, expe
         expect=(
             Path("git_root"),
             {Path("a.py")},
-            set(),
+            Exclusions(black=set(), isort={"**/*"}),
             RevisionRange("HEAD", ":WORKTREE:"),
-            False,
             {},
         ),
     ),
@@ -523,9 +634,8 @@ def test_black_options_skip_magic_trailing_comma(git_repo, config, options, expe
         expect=(
             Path("git_root"),
             {Path("a.py")},
-            set(),
+            Exclusions(black=set(), isort=set()),
             RevisionRange("HEAD", ":WORKTREE:"),
-            True,
             {},
         ),
     ),
@@ -534,9 +644,8 @@ def test_black_options_skip_magic_trailing_comma(git_repo, config, options, expe
         expect=(
             Path("git_root"),
             {Path("a.py")},
-            set(),
+            Exclusions(black=set(), isort={"**/*"}),
             RevisionRange("HEAD", ":WORKTREE:"),
-            False,
             {"config": "my.cfg"},
         ),
     ),
@@ -545,9 +654,8 @@ def test_black_options_skip_magic_trailing_comma(git_repo, config, options, expe
         expect=(
             Path("git_root"),
             {Path("a.py")},
-            set(),
+            Exclusions(black=set(), isort={"**/*"}),
             RevisionRange("HEAD", ":WORKTREE:"),
-            False,
             {"line_length": 90},
         ),
     ),
@@ -556,9 +664,8 @@ def test_black_options_skip_magic_trailing_comma(git_repo, config, options, expe
         expect=(
             Path("git_root"),
             {Path("a.py")},
-            set(),
+            Exclusions(black=set(), isort={"**/*"}),
             RevisionRange("HEAD", ":WORKTREE:"),
-            False,
             {"skip_string_normalization": True},
         ),
     ),
@@ -567,9 +674,8 @@ def test_black_options_skip_magic_trailing_comma(git_repo, config, options, expe
         expect=(
             Path("git_root"),
             {Path("a.py")},
-            set(),
+            Exclusions(black=set(), isort={"**/*"}),
             RevisionRange("HEAD", ":WORKTREE:"),
-            False,
             {},
         ),
     ),
@@ -589,7 +695,9 @@ def test_options(git_repo, options, expect):
         retval = main(options)
 
     expect = (Path(git_repo.root), expect[1]) + expect[2:]
-    format_edited_parts.assert_called_once_with(*expect, report_unmodified=False)
+    format_edited_parts.assert_called_once_with(
+        *expect, report_unmodified=False, workers=1
+    )
     assert retval == 0
 
 
