@@ -6,7 +6,8 @@ import os
 import re
 from datetime import datetime, timedelta
 from pathlib import Path
-from subprocess import DEVNULL, PIPE, CalledProcessError, check_call
+from subprocess import DEVNULL, PIPE, CalledProcessError, check_call  # nosec
+from textwrap import dedent  # nosec
 from typing import List, Union
 from unittest.mock import call, patch
 
@@ -110,7 +111,7 @@ def test_git_get_content_at_revision(git_repo, revision, expect_lines, expect_mt
     if expect_mtime:
         mtime_then = datetime.strptime(result.mtime, GIT_DATEFORMAT)
         difference = expect_mtime() - mtime_then
-        assert timedelta(0) <= difference < timedelta(seconds=3)
+        assert timedelta(0) <= difference < timedelta(seconds=6)
     else:
         assert result.mtime == ""
     assert result.encoding == "utf-8"
@@ -139,6 +140,17 @@ def test_revisionrange_parse(revision_range, expect):
     assert result == expect
 
 
+def git_call(cmd, encoding=None):
+    """Returns a mocked call to git"""
+    return call(
+        cmd.split(),
+        cwd=str(Path("/path")),
+        encoding=encoding,
+        stderr=PIPE,
+        env={"LC_ALL": "C", "PATH": os.environ["PATH"]},
+    )
+
+
 @pytest.mark.kwparametrize(
     dict(
         revision=":WORKTREE:",
@@ -147,31 +159,31 @@ def test_revisionrange_parse(revision_range, expect):
     dict(
         revision="HEAD",
         expect_git_calls=[
-            "git show HEAD:./my.txt",
-            "git log -1 --format=%ct HEAD -- my.txt",
+            git_call("git show HEAD:./my.txt"),
+            git_call("git log -1 --format=%ct HEAD -- my.txt", encoding="utf-8"),
         ],
         expect_textdocument_calls=[
-            call.from_lines([b"1627107028"], mtime="2021-07-24 06:10:28.000000 +0000")
+            call.from_bytes(b"1627107028", mtime="2021-07-24 06:10:28.000000 +0000")
         ],
     ),
     dict(
         revision="HEAD^",
         expect_git_calls=[
-            "git show HEAD^:./my.txt",
-            "git log -1 --format=%ct HEAD^ -- my.txt",
+            git_call("git show HEAD^:./my.txt"),
+            git_call("git log -1 --format=%ct HEAD^ -- my.txt", encoding="utf-8"),
         ],
         expect_textdocument_calls=[
-            call.from_lines([b"1627107028"], mtime="2021-07-24 06:10:28.000000 +0000")
+            call.from_bytes(b"1627107028", mtime="2021-07-24 06:10:28.000000 +0000")
         ],
     ),
     dict(
         revision="master",
         expect_git_calls=[
-            "git show master:./my.txt",
-            "git log -1 --format=%ct master -- my.txt",
+            git_call("git show master:./my.txt"),
+            git_call("git log -1 --format=%ct master -- my.txt", encoding="utf-8"),
         ],
         expect_textdocument_calls=[
-            call.from_lines([b"1627107028"], mtime="2021-07-24 06:10:28.000000 +0000")
+            call.from_bytes(b"1627107028", mtime="2021-07-24 06:10:28.000000 +0000")
         ],
     ),
     expect_git_calls=[],
@@ -189,17 +201,7 @@ def test_git_get_content_at_revision_obtain_file_content(
 
         git.git_get_content_at_revision(Path("my.txt"), revision, Path("/path"))
 
-        expected_calls = [
-            call(
-                expected_call.split(),
-                cwd=str(Path("/path")),
-                encoding="utf-8",
-                stderr=PIPE,
-                env={"LC_ALL": "C", "PATH": os.environ["PATH"]},
-            )
-            for expected_call in expect_git_calls
-        ]
-        assert check_output.call_args_list == expected_calls
+        assert check_output.call_args_list == expect_git_calls
         assert text_document_class.method_calls == expect_textdocument_calls
 
 
@@ -248,9 +250,9 @@ def test_revisionrange_parse_with_common_ancestor(git_repo, revrange, expect):
     dict(path="file.12345.tmp", expect="file.12345.tmp"),
     dict(path="subdir/file.12345.tmp", expect="subdir/file.12345.tmp"),
 )
-def test_get_rev1_path(path, expect):
-    """``get_rev1_path`` drops two suffixes from ``.py.<HASH>.tmp``"""
-    result = git.get_rev1_path(Path(path))
+def test_get_path_in_repo(path, expect):
+    """``get_path_in_repo`` drops two suffixes from ``.py.<HASH>.tmp``"""
+    result = git.get_path_in_repo(Path(path))
 
     assert result == Path(expect)
 
@@ -403,6 +405,58 @@ def test_git_get_content_at_revision_stderr(git_repo, capfd, caplog):
     assert caplog.text == ""
 
 
+@pytest.fixture(scope="module")
+def encodings_repo(tmp_path_factory):
+    """Create an example Git repository using various encodings for the same file"""
+    tmpdir = tmp_path_factory.mktemp("branched_repo")
+    git_repo = GitRepoFixture.create_repository(tmpdir)
+    # Commit without an encoding cookie, defaults to utf-8
+    git_repo.add({"file.py": "darker = 'plus foncé'\n"}, commit="Default encoding")
+    git_repo.create_tag("default")
+    # Commit without an encoding cookie but with a utf-8 signature
+    content = "darker = 'plus foncé'\n".encode("utf-8-sig")
+    git_repo.add({"file.py": content}, commit="utf-8-sig")
+    git_repo.create_tag("utf-8-sig")
+    # Commit with an iso-8859-1 encoding cookie
+    content = "# coding: iso-8859-1\ndarker = 'plus foncé'\n".encode("iso-8859-1")
+    git_repo.add({"file.py": content}, commit="iso-8859-1")
+    git_repo.create_tag("iso-8859-1")
+    # Commit with a utf-8 encoding cookie
+    content = "# coding: utf-8\npython = 'パイソン'\n".encode()
+    git_repo.add({"file.py": content}, commit="utf-8")
+    git_repo.create_tag("utf-8")
+    # Current worktree content (not committed) with a shitfjs encoding cookie
+    content = "# coding: shiftjis\npython = 'パイソン'\n".encode("shiftjis")
+    git_repo.add({"file.py": content})
+    return git_repo
+
+
+@pytest.mark.kwparametrize(
+    dict(commit="default", encoding="utf-8", lines=("darker = 'plus foncé'",)),
+    dict(commit="utf-8-sig", encoding="utf-8-sig", lines=("darker = 'plus foncé'",)),
+    dict(
+        commit="iso-8859-1",
+        encoding="iso-8859-1",
+        lines=("# coding: iso-8859-1", "darker = 'plus foncé'"),
+    ),
+    dict(
+        commit="utf-8", encoding="utf-8", lines=("# coding: utf-8", "python = 'パイソン'")
+    ),
+    dict(
+        commit=":WORKTREE:",
+        encoding="shiftjis",
+        lines=("# coding: shiftjis", "python = 'パイソン'"),
+    ),
+)
+def test_git_get_content_at_revision_encoding(encodings_repo, commit, encoding, lines):
+    """Git file is loaded using its historical encoding"""
+    result = git.git_get_content_at_revision(
+        Path("file.py"), commit, encodings_repo.root
+    )
+    assert result.encoding == encoding
+    assert result.lines == lines
+
+
 @pytest.mark.kwparametrize(
     dict(retval=0, expect=True),
     dict(retval=1, expect=False),
@@ -803,6 +857,51 @@ def test_edited_linenums_differ_revision_vs_lines(git_repo, context_lines, expec
     assert linenums == expect
 
 
+@pytest.mark.kwparametrize(
+    dict(context_lines=0, expect=[1, 3, 4, 5, 6, 8]),
+    dict(context_lines=1, expect=[1, 2, 3, 4, 5, 6, 7, 8]),
+)
+def test_edited_linenums_differ_revision_vs_lines_multiline_strings(
+    git_repo, context_lines, expect
+):
+    """Tests for EditedLinenumsDiffer.revision_vs_lines() with multi-line strings"""
+    git_repo.add(
+        {
+            "a.py": dedent(
+                """\
+                change\n
+                keep\n
+                '''change first,\n
+                keep second\n
+                and third,\n
+                change fourth line of multiline'''\n
+                keep\n
+                change\n
+                """
+            )
+        },
+        commit="Initial commit",
+    )
+    content = TextDocument.from_lines(
+        [
+            "CHANGED",
+            "keep",
+            "'''CHANGED FIRST,",
+            "keep second",
+            "and third,",
+            "CHANGED FOURTH LINE OF MULTILINE'''",
+            "keep",
+            "CHANGED",
+        ]
+    )
+    revrange = git.RevisionRange("HEAD", ":WORKTREE:")
+    differ = git.EditedLinenumsDiffer(git_repo.root, revrange)
+
+    linenums = differ.revision_vs_lines(Path("a.py"), content, context_lines)
+
+    assert linenums == expect
+
+
 def test_local_gitconfig_ignored_by_gitrepofixture(tmp_path):
     """Tests that ~/.gitconfig is ignored when running darker's git tests"""
     (tmp_path / "HEAD").write_text("ref: refs/heads/main")
@@ -810,7 +909,7 @@ def test_local_gitconfig_ignored_by_gitrepofixture(tmp_path):
     # Note: once we decide to drop support for git < 2.28, the HEAD file
     # creation above can be removed, and setup can simplify to
     # check_call("git config --global init.defaultBranch main".split())
-    check_call(
+    check_call(  # nosec
         "git config --global init.templateDir".split() + [str(tmp_path)],
         env={"HOME": str(tmp_path), "PATH": os.environ["PATH"]},
     )
