@@ -8,15 +8,16 @@ from argparse import ArgumentError
 from pathlib import Path
 from textwrap import dedent
 from types import SimpleNamespace
-from unittest.mock import Mock, call, patch
+from unittest.mock import call, patch
 
 import pytest
 
 import darker.__main__
 import darker.import_sorting
 import darker.linting
+from darker.config import Exclusions
 from darker.exceptions import MissingPackageError
-from darker.git import WORKTREE, RevisionRange
+from darker.git import WORKTREE, EditedLinenumsDiffer, RevisionRange
 from darker.tests.helpers import isort_present
 from darker.utils import TextDocument, joinlines
 from darker.verification import NotEquivalentError
@@ -123,38 +124,33 @@ A_PY_DIFF_BLACK_ISORT = [
 
 @pytest.mark.kwparametrize(
     dict(
-        enable_isort=False,
-        black_config={},
-        black_exclude=set(),
+        isort_exclude={"**/*"},
         expect=[A_PY_BLACK],
     ),
     dict(
-        enable_isort=True,
-        black_config={},
-        black_exclude=set(),
         expect=[A_PY_BLACK_ISORT],
     ),
     dict(
-        enable_isort=False,
         black_config={"skip_string_normalization": True},
-        black_exclude=set(),
+        isort_exclude={"**/*"},
         expect=[A_PY_BLACK_UNNORMALIZE],
     ),
     dict(
-        enable_isort=False,
         black_exclude={Path("a.py")},
+        isort_exclude={"**/*"},
         expect=[],
     ),
     dict(
-        enable_isort=True,
         black_exclude={Path("a.py")},
         expect=[A_PY_ISORT],
     ),
     black_config={},
+    black_exclude=set(),
+    isort_exclude=set(),
 )
 @pytest.mark.parametrize("newline", ["\n", "\r\n"], ids=["unix", "windows"])
 def test_format_edited_parts(
-    git_repo, enable_isort, black_config, black_exclude, newline, expect
+    git_repo, black_config, black_exclude, isort_exclude, newline, expect
 ):
     """Correct reformatting and import sorting changes are produced
 
@@ -170,9 +166,8 @@ def test_format_edited_parts(
     result = darker.__main__.format_edited_parts(
         Path(git_repo.root),
         {Path("a.py")},
-        black_exclude,
+        Exclusions(black=black_exclude, isort=isort_exclude),
         RevisionRange("HEAD", ":WORKTREE:"),
-        enable_isort,
         black_config,
         report_unmodified=False,
     )
@@ -204,9 +199,8 @@ def test_format_edited_parts_all_unchanged(git_repo, monkeypatch):
         darker.__main__.format_edited_parts(
             Path(git_repo.root),
             {Path("a.py"), Path("b.py")},
-            set(),
+            Exclusions(black=set(), isort=set()),
             RevisionRange("HEAD", ":WORKTREE:"),
-            True,
             {},
             report_unmodified=False,
         )
@@ -228,9 +222,8 @@ def test_format_edited_parts_ast_changed(git_repo, caplog):
             darker.__main__.format_edited_parts(
                 git_repo.root,
                 {Path("a.py")},
-                set(),
+                Exclusions(black=set(), isort={"**/*"}),
                 RevisionRange("HEAD", ":WORKTREE:"),
-                enable_isort=False,
                 black_config={},
                 report_unmodified=False,
             )
@@ -273,9 +266,8 @@ def test_format_edited_parts_isort_on_already_formatted(git_repo):
     result = darker.__main__.format_edited_parts(
         git_repo.root,
         {Path("a.py")},
-        set(),
+        Exclusions(black=set(), isort=set()),
         RevisionRange("HEAD", ":WORKTREE:"),
-        enable_isort=True,
         black_config={},
         report_unmodified=False,
     )
@@ -329,9 +321,8 @@ def test_format_edited_parts_historical(git_repo, rev1, rev2, expect):
     result = darker.__main__.format_edited_parts(
         git_repo.root,
         {Path("a.py")},
-        set(),
+        Exclusions(black=set(), isort=set()),
         RevisionRange(rev1, rev2),
-        enable_isort=True,
         black_config={},
         report_unmodified=False,
     )
@@ -373,7 +364,7 @@ def test_format_edited_parts_historical(git_repo, rev1, rev2, expect):
     black_config={},
     expect="import  original\nprint( original )\n",
 )
-def test_reformat_single_file(
+def test_blacken_single_file(
     git_repo,
     relative_path,
     rev1,
@@ -384,14 +375,15 @@ def test_reformat_single_file(
     black_config,
     expect,
 ):
-    """Test for ``_reformat_single_file``"""
+    """Test for ``_blacken_single_file``"""
     git_repo.add(
         {"file.py": "import  original\nprint( original )\n"}, commit="Initial commit"
     )
-    result = darker.__main__._reformat_single_file(
+    result = darker.__main__._blacken_single_file(
         git_repo.root,
         Path(relative_path),
-        RevisionRange(rev1, rev2),
+        Path("file.py"),
+        EditedLinenumsDiffer(git_repo.root, RevisionRange(rev1, rev2)),
         TextDocument(rev2_content),
         TextDocument(rev2_isorted),
         enable_isort,
@@ -578,6 +570,7 @@ def test_main_in_plain_directory(tmp_path, capsys):
                 Path("subdir_b/subdir_c/another python file.py"),
             },
             RevisionRange(rev1="HEAD", rev2=":WORKTREE:"),
+            False,
         )
     ]
     output = capsys.readouterr().out
@@ -629,6 +622,21 @@ def test_main_historical(git_repo):
         darker.__main__.main(["--revision=foo..bar", "."])
 
 
+@pytest.mark.parametrize("arguments", [["--diff"], ["--check"], ["--diff", "--check"]])
+@pytest.mark.parametrize("src", [".", "foo/..", "{git_repo_root}"])
+def test_main_historical_ok(git_repo, arguments, src):
+    """Runs ok for repository root with rev2 specified and ``--diff`` or ``--check``"""
+    git_repo.add({"README": "first"}, commit="Initial commit")
+    initial = git_repo.get_hash()
+    git_repo.add({"README": "second"}, commit="Second commit")
+    second = git_repo.get_hash()
+
+    darker.__main__.main(
+        arguments
+        + [f"--revision={initial}..{second}", src.format(git_repo_root=git_repo.root)]
+    )
+
+
 def test_main_pre_commit_head(git_repo, monkeypatch):
     """Warn if run by pre-commit, rev2=HEAD and no ``--diff`` or ``--check`` provided"""
     git_repo.add({"a.py": "original = 1"}, commit="Add a.py")
@@ -696,12 +704,11 @@ def test_main_vscode_tmpfile(git_repo, capsys):
     ]
 
 
-def test_print_diff(tmp_path, monkeypatch, capsys):
+def test_print_diff(tmp_path, capsys):
     """print_diff() prints Black-style diff output with 5 lines of context"""
-    monkeypatch.chdir(tmp_path)
-    Path("a.py").write_text("dummy\n")
+    Path(tmp_path / "a.py").write_text("dummy\n", encoding="utf-8")
     darker.__main__.print_diff(
-        Path("a.py"),
+        Path(tmp_path / "a.py"),
         TextDocument.from_lines(
             [
                 "unchanged",
@@ -740,6 +747,8 @@ def test_print_diff(tmp_path, monkeypatch, capsys):
             ],
             mtime="2020-10-08 19:21:09.005501 +0000",
         ),
+        root=tmp_path,
+        use_color=False,
     )
 
     assert capsys.readouterr().out.splitlines() == [
@@ -790,45 +799,33 @@ def test_modify_file(tmp_path, new_content, expect):
 @pytest.mark.kwparametrize(
     dict(
         new_content=TextDocument(lines=['print("foo")']),
-        tty=False,
-        with_pygments=False,
+        use_color=False,
         expect=('print("foo")\n',),
     ),
     dict(
         new_content=TextDocument(lines=['print("foo")']),
-        tty=False,
-        with_pygments=True,
+        use_color=False,
         expect=('print("foo")\n',),
     ),
     dict(
         new_content=TextDocument(lines=['print("foo")']),
-        tty=True,
-        with_pygments=False,
-        expect=('print("foo")\n',),
-    ),
-    dict(
-        new_content=TextDocument(lines=['print("foo")']),
-        tty=True,
-        with_pygments=True,
+        use_color=True,
         expect=(
             '\x1b[36mprint\x1b[39;49;00m(\x1b[33m"\x1b[39;49;00mfoo'
             + '\x1b[33m"\x1b[39;49;00m)\n',
             '\x1b[36mprint\x1b[39;49;00m(\x1b[33m"\x1b[39;49;00m\x1b[33mfoo'
             + '\x1b[39;49;00m\x1b[33m"\x1b[39;49;00m)\n',
+            # Pygments 2.4.x:
+            '\x1b[34mprint\x1b[39;49;00m(\x1b[33m"\x1b[39;49;00mfoo'
+            + '\x1b[33m"\x1b[39;49;00m)\n',
         ),
     ),
 )
-def test_print_source(new_content, tty, with_pygments, expect, capsys):
-    """Highlight is applied only if tty, final newline is handled correctly."""
-    with patch("sys.stdout.isatty", Mock(return_value=tty)), patch(
-        "darker.__main__._import_pygments",
-        Mock(
-            return_value=darker.__main__._import_pygments(),
-            side_effect=None if with_pygments else ImportError(),
-        ),
-    ):
-        darker.__main__.print_source(new_content)
-        assert capsys.readouterr().out in expect
+def test_print_source(new_content, use_color, expect, capsys):
+    """Highlight is applied only if specified, final newline is handled correctly."""
+    darker.__main__.print_source(new_content, use_color=use_color)
+
+    assert capsys.readouterr().out in expect
 
 
 def test_stdout_path_resolution(git_repo, capsys):
